# A package used for Atlas assembly
#
# AUTHOR: Ynaru Ren (yanrur@bcm.tmc.edu)
# 
#
########################################

package Atlas::AsmWgs;

require Exporter;
@ISA = qw( Exporter );
@EXPORT = (qw(
	      $OVERLAPPER_EXE
	      $BINNER_EXE
	      $EXTRACT_EXE
	      $SCAFFOLD_EXE
	      $BIN_ASM_EXE
	      $LINEAR_EXE
	      $PHRAP_EXE
	      run_overlapper
	      read_in_fonfile
	      mk_bin_dirs
	      extract_bin_file
	      run_asm_bin
	      join_ace_files
	      join_contigs
              ));

use Atlas::PrepReads;

## variables
$OVERLAPPER_EXE = $ENV{"ATLAS_ROOT"}."/bin/atlas-overlapper";
$BINNER_EXE = $ENV{"ATLAS_ROOT"}."/bin/atlas-binner";
$EXTRACT_EXE = $ENV{"ATLAS_ROOT"}."/bin/atlas-extractbins";
$SCAFFOLD_EXE = $ENV{"ATLAS_ROOT"}."/perl/bin/atlas-build-scaffold-file";
$BIN_ASM_EXE = $ENV{"ATLAS_ROOT"}."/perl/bin/atlas-separate-bin-assemble";
$LINEAR_EXE = $ENV{"ATLAS_ROOT"}."/bin/atlas-linearsequence";
#You need to edit the following variables to reflect where your executables are
$PHRAP_EXE = $ENV{"ATLAS_ROOT"}."/local/phrap";

#subroutines
sub run_overlapper {
    my ($source_fof, $query_fof, $parallel, @opts) = @_;
    my @source;
#    my $header = " ovlp-";
    my $options = join(" ",  @opts);
    my $num, $graph_file;
    open SO, $source_fof||die "cannot open $source_fof\n";
    while (<SO>){
        chomp $_;
        push @source, $_;
    }
    close SO;
    my $i=0;
    foreach my $source_file (@source){
        $i++;
        if ($source_file =~ /.+\/(.+)\.gz/ ){
            $graph_file = "$1.graph";
            $log_file = "$1.overlapper.stdout";
        }
        my $cmd = $OVERLAPPER_EXE." -s $source_file -o $graph_file -Q $query_fof -b 0 -e 30000 ".$options." >$log_file 2>&1" ;
#	print $cmd,"\n";
	&submit_job ($cmd, $parallel)
    }
    if ($parallel){
        &lsf_wait();
    }

    #put them together
    my $final_graph_file = "atlas.graph";
    my $cat_cmd = "cat *graph >$final_graph_file";
    system $cat_cmd;
    #cleanup
    my @graph = glob("*graph");
    foreach my $g ( @graph ){
        chomp $g;
        unless ($g eq $final_graph_file){
            print "delete $g\n";
            unlink $g;
        }
    }
    system ("rm -f *overlapper.stdout");
    return; 
}

sub read_in_fonfile {
    my ($fonfile, $begin, $end) = @_;
    my $sub_dir_no1 = 10000;
    my $sub_dir_no2 = 100;
    my %bins = ();
    open FON, "$fonfile" or die "Cannot open $fonfile\n";
    while (<FON>){
        if($_ =~ /^(\d+)\s+(\S+)/) {
            my $bin= $1; 
            my $read= $2;
            next if ($bin < $begin);
            last if ($bin > $end);
            my $sub1 = int($bin/$sub_dir_no1);
            my $sub2 = int(($bin%$sub_dir_no1)/$sub_dir_no2);
            if ($sub2 =~/^\d$/){
                $sub2 = "0".$sub2;
            }
            push @{$bins{$sub1}{$sub2}{$bin}}, $read;
        }
    }
    close FON;
    return %bins;
}

sub mk_bin_dirs {
    ($falist, %bins) = @_;
    foreach my $sub1 (sort {$a<=>$b} keys %bins) {
        system("mkdir $sub1") if (!-e $sub1);
        chdir($sub1);
        foreach my $sub2 (sort {$a<=>$b} keys %{$bins{$sub1}}) {
            system("mkdir $sub2") if (!-e $sub2);
            chdir($sub2);
            foreach my $bin (sort {$a<=>$b} keys %{$bins{$sub1}{$sub2}}) {
                system("mkdir $bin") if (!-e $bin);
                chdir($bin);
                print "$sub1\t$sub2\t$bin\n";
                my $binfile =  "bin.$bin";
                system("rm -f *"); #remove old results
                open OUT, ">$binfile" or die "Cannot open $binfile\n";
                foreach my $read (@{$bins{$sub1}{$sub2}{$bin}}) {
                    print OUT "$read\n";
                }
                close(OUT);
                if (-s $binfile){
                    my ($fa, $qual) = &extract_bin_file ($binfile, $falist);
		    unless (-s $fa and -s $qual){
                        print "ERR: extract reads failed for $binfile.\n";
                        return;
                    }
                    &run_asm_bin ($fa, $qual);
                    
                    if (-s $binfile."_out.fa.ace"){
                        print "assmebly done for $binfile\n";
                    } else {
                        print "assembly failed for $binfile\n";
                    }
                }else{
                    print "no $binfile or $binfile empty....\n";
                    next;
                }
                chdir("..");
            }
            chdir("..");
        }
        chdir("..");
    }
}

sub extract_bin_file {
    my ($binfile, $falist) = @_;
    system ("$EXTRACT_EXE -i $falist $binfile >/dev/null");
    my $fa = $binfile."_out.fa";
    my $qual = $fa.".qual";
    return $fa, $qual;
}

sub run_asm_bin {
    my ($fa, $qual) = @_;
    system ("$CROSSMATCH_EXE $fa $screen_file -minmatch 12 -minscore 20 -penalty -2 -screen > $fa.screen.stdout");
    if (!-e "$fa.screen"){
	die "Failed on cross_match\n";
    }
    rename ($fa, $fa."_pre_screen.fa");
    rename ("$fa.screen", $fa);
    my $cmd = "$WINDOW_EXE -s v -l 50 +x -O _pre_screen.fa -Q .qual -q  $fa\n";
    system ($cmd);
    unless (-s "$fa.pass"){
        print "ERR: triming failed for $binfile\n";
        return;
    }
    rename $fa, $fa."_pre-window.fa";
    rename "$fa.pass", $fa;
 #   system ("$PHRAP_EXE $fa -ace -forcelevel 10 >phrap.stdout");
system ("$PHRAP_EXE $fa -ace  >phrap.stdout");
    return;
}

sub join_ace_files {
    my ($ace_out, $ace_list) = @_;
    my $num_contigs = 0;
    my $num_reads = 0;
    my $contigs, $reads;
    my @ace_list;
    open LIST, $ace_list||die "cannot open $ace_list\n";
    @ace_list = <LIST>;
    close LIST;
    
    open OUT, ">$ace_out"||die "cannot open $ace_out\n";
    print "sum up number of contigs and reads......\n";
    foreach my $a ( @ace_list){
        chomp $a;
        open ACE, $a ||die "cannot open $a\n";
        foreach my $line (<ACE>){
            if ($line =~ /^AS\s+(\d+)\s+(\d+)$/) {
                $contigs = $1;
                $reads = $2;
                last;
            }
        }
        $num_contigs += $contigs;
        $num_reads += $reads;
        close ACE;
    }
    print OUT "AS $num_contigs $num_reads\n";
    print "AS $num_contigs $num_reads\n";

    foreach my $a (@ace_list){
        chomp $a;
        if ($a =~ /bin\.(\d+)_out/){
            $bin = $1;
        }
        open ACE, $a ||die "cannot open $a\n";
        foreach my $line (<ACE>){
            if ($line =~ /^AS\s/){
                next;
            } elsif ( $line =~ /^CO Contig\d/){
                $line =~ s/Contig/Contig$bin-/;
                print OUT $line;
                print $line;
            }else {
                print OUT $line;
            }
        }
        close ACE;
    }
    close OUT;
}

sub join_contigs {
    my ($contig_out, $ace_list) = @_;
    my @contig_list = ();

    if (-e $contig_out or -e "$contig_out.qual") {
        system ("rm -f $contig_out $contig_out.qual");
    }
    open LIST, $ace_list||die "cannot open $ace_list\n";
    foreach my $a (<LIST>){
        chomp $a;
        my $s = $a;
        $s =~ s/ace/contigs/;
        push @contig_list, $s;
    }
    close LIST;

    open OUT, ">$contig_out"||die "cannot open $contig_out \n";
    open OUT1, ">$contig_out.qual"||die "cannot open $contig_out.qual \n";
    foreach my $seq (@contig_list){
        open CON, $seq ||die "cannot open $seq\n";
        foreach $line (<CON>){
            if ($line =~ /^>bin\.(\d+)_out\.fa\.Contig(\d+)/){
                $line = ">Contig$1-$2 $seq \n";
            }
            print OUT $line;
        }
        close CON||die "cannot properly close $seq\n";
        my $qual = "$seq.qual";
        open CON, $qual ||die "cannot open $qual\n";
        foreach $line (<CON>){
            if ($line =~ /^>bin\.(\d+)_out\.fa\.Contig(\d+)/){
                $line = ">Contig$1-$2 \n";
            }
            print OUT1 $line;
        }
        close CON||die "cannot properly close $seq\n";
    }
    close OUT;
    close OUT1;
    return;
}

#END
