#  A package used to prepare reads files for atlas 
#  assembly
#  
#  AUTHOR: Ynaru Ren (yanrur@bcm.tmc.edu)
#
########################################

package Atlas::PrepReads;

require Exporter;

@ISA = qw( Exporter );

@EXPORT = (qw(
	      $screen_file
	      $CREATE_INDEX_EXE
	      $DIVIDE_FAFILE_EXE
	      $WINDOW_EXE
	      $CROSSMATCH_EXE
	      $SCREEN_TRIM_EXE
	      $MER_COUNT
	      submit_job
	      run_parallel
	      parallel_wait
	      run_mer_count
	      find_kmer_peak
	      eliminate_low_freq
              ));

## variables

$screen_file = $ENV{"ATLAS_ROOT"}."/data/univec/univec.fa";
$CREATE_INDEX_EXE = $ENV{"ATLAS_ROOT"}."/perl/bin/atlas-createindex";
$DIVIDE_FAFILE_EXE = $ENV{"ATLAS_ROOT"}."/perl/bin/atlas-divide-fafile";
$WINDOW_EXE = $ENV{"ATLAS_ROOT"}."/bin/atlas-screen-window";
$SCREEN_TRIM_EXE = $ENV{"ATLAS_ROOT"}."/perl/bin/atlas-screen-trim-file";
$MER_COUNT = $ENV{"ATLAS_ROOT"}."/bin/atlas-count-kmers";

#You need to edit the following variables to reflect where your executables are
$CROSSMATCH_EXE = $ENV{"ATLAS_ROOT"}."/local/cross_match";

#subroutines
# 
sub submit_job{
    my ($cmd, $parallel) = @_;
    if ($parallel){
	&run_parallel ();
    }else{
	print $cmd,"\n";
	system ($cmd);
    }
    return;
}

#You need to write this to fit your system	
sub run_parallel{
    return;
}

#You need to write this to fit your system
sub parallel_wait{
    return;
}

sub run_mer_count {
    my ($k, $slice, $hashsize, $minreport, $parallel) = @_;
    my $j, $cmd;
    for ($j=0; $j<$slice; $j++){
	$cmd = "gunzip -c ../*fa.pass.gz|$MER_COUNT $k $hashsize $slice $j $minreport >mer.$j.results";
	&submit_job ($cmd, $parallel);
    }
    if ($parallel){
	&parallel_wait;
    }
    return;
}

sub find_kmer_peak {
    my ($distrib, $cut_off_init) = @_;
    open DIS, "$distrib"||die "cannot open $distrib\n";
    my %freq = ();
    while (<DIS>){
        chomp $_;
        my ($key, $value) = split;
        if ($key > 2 and $key < 12){
            $freq{$key} = $value;
        }
    }
    
    my $cut_off = $cut_off_init; #default
    for ($key=3; $key<=10; $key++){
        if (exists $freq{$key}){
            if (($freq{$key+1} > $freq{$key}) and ($freq{$key+2} < $freq{$key+1})){
                $cut_off = 3*($key+1);
                last;
            }
        }
    }
    return $cut_off;
}
sub eliminate_low_freq {
    my ($cut, $out_file) = @_;
    my @mer_files = glob ("*results");
    my $input, $freq;
    open OUT, ">$out_file"||die "cannot open $out_file\n";

    foreach $mer_file (@mer_files){

        if ($mer_file =~ /\.gz/){
            $input = "gunzip -c $mer_file|";
        }else{
            $input = $mer_file;
        }
   
        open IN, $input ||die "cannot open $input\n";
        print "process $input................\n";
        while (<IN>){
            if ($_ =~/^[acgt]+\s+(\d+)$/){
                $freq = $1;
                if ($freq >= $cut){
                    print OUT $_;
                }elsif($freq < $cut){
                    next;
                }
            }
        }
        close IN;
    }
    close OUT;  
    return;
}
  

#END
